#ifndef OBJ05_DB_API_H
#define OBJ05_DB_API_H

/* RDM 11.0.1.551 */

#include "Db.h"
#include "Cursor.h"
#include "obj05_db.h"

namespace RDM_CPP {

class Cursor_person;

/*! \brief The obj05_db Db class
 *
 * This class provides the implementation for the obj05_db database
*/
class Db_obj05_db : public Db {
public:
    /*! \brief Open the obj05_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj05_db object
     */
    static Db_obj05_db Open (char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj05_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj05_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj05_db Open (wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    /*! \brief Open the obj05_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj05_db object
     */
    static Db_obj05_db Open (const char *name, const char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj05_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj05_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj05_db Open (const wchar_t *name, const wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    //! Db_obj05_db Default Constructor
    Db_obj05_db (void) throw ();
    /*! \brief Db_obj05_db Constructor
     *
     * \param db A Db object used to create the Db_obj05_db
     */
    Db_obj05_db (const Db& db) throw ();
    //! Db_obj05_db Copy Constructor
    Db_obj05_db (const Db_obj05_db&) throw ();

    //! Db_obj05_db assignment operator
    Db_obj05_db& operator = (const Db&) throw (rdm_exception&);

    //! Db_obj05_db assignment operator
    Db_obj05_db& operator = (const Db_obj05_db&) throw ();

    /*! \brief Db_obj05_db destructor
     *
     * Remove a reference to the underlying database.  The database is closed when the last reference is gone.
     */
    ~Db_obj05_db ();

    /*! \brief Create a new person record
     *
     * \returns A singleton Cursor_person object with default field values
     *
     * Create a new person record using default values
     */
    Cursor_person New_person_record (void) const throw (rdm_exception&);

    /*! \brief Create a new person record
     *
     * \param fields The field values for the new person record
     * \returns A singleton Cursor_person object with the specified field values
     *
     * Create a new person record using the specified values
     */
    Cursor_person New_person_recordWithFieldValues (struct person *fields) const throw (rdm_exception&);

    /*! \brief Obtain person cursor position at the first person record
     * \returns person cursor positioned at the first person record
     *
     * Create a new person cursor for doing a table scan of the person records.
     * The cursor is positioned at the first person record
     */
    Cursor_person Get_person_records (void) const throw (rdm_exception&);

    /*! \brief Obtain a person cursor based on the last_four_ssn key
     * \returns person cursor positioned at the first person last_four_ssn key value
     *
     * Create a new person cursor based on the last_four_ssn key and position the cursor
     * at the first record
     */
    Cursor_person Get_person_recordsBy_last_four_ssn () const throw (rdm_exception&);

private:
    /*! \brief Db_obj05_db reference operator */
    Db_obj05_db* operator & (void);
    /*! \brief Db_obj05_db constructor */
    Db_obj05_db (IDb *idb);
    /*! \brief Get the Db_obj05_db dbType
     * \return The Db_obj05_db dbType
     */
    static dbType Type (void);
};

/*! \brief The person Cursor class
 *
 * This class provides the implementation for the person Cursor
*/
class Cursor_person : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_person objects
     */
    static recordType Type (void);

    /*! Cursor_person Default Constructor */
    Cursor_person (void) throw ();
    /*! \brief Cursor_person Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_person object
     */
    Cursor_person (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_person Copy Constructor */
    Cursor_person (const Cursor_person&) throw ();

    /*! \brief Cursor_person assignment operator */
    Cursor_person& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_person assignment operator */
    Cursor_person& operator = (const Cursor_person& cursor) throw ();

    /*! \brief Create a new person cursor doing last_four_ssn key navigation.
     *
     * \return person cursor ordered by the last_four_ssn key
     *
     *  Create a new person cursor using last_four_ssn key navigation and position the
     *  new cursor at our current position
     */
    Cursor_person GetRecordsBy_last_four_ssn () const throw (rdm_exception&);

    /*! \brief Position the person cursor to the last_four_ssn key value
     *
     * \param last_four_ssn The last_four_ssn value to position the cursor at
     *
     */
    void KeyFindBy_last_four_ssn (char last_four_ssn_val[5]) const throw (rdm_exception&);

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct person *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct person *fields) const throw (rdm_exception&);

    /*! \brief Set the last_four_ssn field value for the current record
     * \param last_four_ssn The field value that will be written to the record at the current cursor position
     */
    void Set_last_four_ssn (const char last_four_ssn_val[5]) const throw (rdm_exception&);

    /*! \brief Get the last_four_ssn field value for the current record
     * \param last_four_ssn A pointer where the field value from the current cursor position will be written
     */
    void Get_last_four_ssn (char last_four_ssn_val[5]) const throw (rdm_exception&);

    /*! \brief Cursor_person Destructor
     *
     * Release the cursor
     */
    ~Cursor_person ();

private:
    friend class Db_obj05_db;
};

}
#endif    /* OBJ05_DB_API_H */
